; ==================================================================================================
; Title:   SetPrivilegeToken.asm
; Author:  Rewritten by Biterirder for ObjMem32 library. 
; Version: 1.0.0
; Purpose: Enabling/disabling of process privilege tokens. Requires AdvAPI32 library.
;          Version 1.0.0, November 2006
;            - First release.
; ==================================================================================================


; 
; Procedure: SetPrivilegeToken
; Purpose:   Enabling of provilege tokens.
; Arguments: Arg1: Process handle.
;            Arg2: -> Privilege name (ANSI string).
;            Arg3: Eanble = TRUE, disable = FALSE
; Return:    Zero if failed.

SetPrivilegeToken proc hProcess:Handle, pPrivilegeName:Pointer, dEnable:dword
    local TokenPriv:TOKEN_PRIVILEGES, PrevTokenPriv:TOKEN_PRIVILEGES
    local luid:LUID, hToken:Handle, dReturnLenght:dword

    mov dReturnLenght, sizeof TOKEN_PRIVILEGES

    ;Get debug privilege
    invoke OpenProcessToken, hProcess, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, addr hToken
    .if eax != 0
      invoke LookupPrivilegeValue, NULL, pPrivilegeName, addr luid    ;Get LUID on local system
      .if eax != 0
        ;Get current privilege setting
        mov TokenPriv.PrivilegeCount, 1
        m2m TokenPriv.Privileges.Luid.LowPart, luid.LowPart, eax
        m2m TokenPriv.Privileges.Luid.HighPart, luid.HighPart, eax
        mov TokenPriv.Privileges.Attributes, 0
        int 3
        invoke AdjustTokenPrivileges, hToken, FALSE, addr TokenPriv, sizeof TokenPriv, \
                                      addr PrevTokenPriv, addr dReturnLenght
        .if eax != 0
          ;Set privilege based on previous setting
          mov PrevTokenPriv.PrivilegeCount, 1
          m2m PrevTokenPriv.Privileges.Luid.LowPart, luid.LowPart, eax
          m2m PrevTokenPriv.Privileges.Luid.HighPart, luid.HighPart, eax
          .if dEnable != FALSE
            BitSet PrevTokenPriv.Privileges.Attributes, SE_PRIVILEGE_ENABLED
          .else
            BitClr PrevTokenPriv.Privileges.Attributes, SE_PRIVILEGE_ENABLED
          .endif
          invoke AdjustTokenPrivileges, hToken, FALSE, addr PrevTokenPriv, dReturnLenght, NULL, NULL
        .endif
      .endif
      push eax                                ;Save result value
      invoke CloseHandle, hToken              ;Close the handle
      pop eax                                 ;Restore result value
    .endif
    ret
SetPrivilegeToken endp